import { ref, defineComponent, openBlock, createElementBlock, normalizeClass, unref, Fragment, renderList, createBlock, resolveDynamicComponent } from "vue";
import { dataBindingBuilder, createRegistrations } from "@supermatic-ui/core";
const setupTemplate = (dataBinding, template, vueRef) => {
  const useRef = vueRef || ref("");
  const reactivity = dataBinding.evaluateTemplateReactive(template || "");
  useRef.value = reactivity.value;
  reactivity.subscribe((value) => {
    useRef.value = value;
  });
  return useRef;
};
const setupEval = (dataBinding, expression, vueRef) => {
  const useRef = vueRef || ref();
  const reactivity = dataBinding.evaluateReactive(expression || "");
  useRef.value = reactivity.value;
  reactivity.subscribe((value) => {
    useRef.value = value;
  });
  return useRef;
};
const setupBind = (dataBinding, binding, vueRef) => {
  if (binding) {
    const property = dataBinding.evaluateTemplate(binding);
    if (property != null) {
      const reactivity = dataBinding.evaluateReactive(property);
      vueRef.value = reactivity.value;
      reactivity.subscribe((value) => {
        vueRef.value = value;
      });
      return {
        value: reactivity.value,
        set: (value) => {
          reactivity.updateValue(value);
        }
      };
    }
  }
  return {
    value: vueRef.value,
    set: () => {
    }
  };
};
const _sfc_main$1 = /* @__PURE__ */ defineComponent({
  __name: "SupermaticContainer",
  props: {
    metadata: null,
    dataBinding: null
  },
  setup(__props) {
    const props = __props;
    const wrapClassName = setupTemplate(props.dataBinding, props.metadata.container.wrapClassName);
    const className = setupTemplate(props.dataBinding, props.metadata.className);
    let components2 = props.dataBinding.registrations.getComponentRegistrations();
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        class: normalizeClass(unref(className))
      }, [
        (openBlock(true), createElementBlock(Fragment, null, renderList(props.metadata.container.components, (item, index) => {
          return openBlock(), createElementBlock(Fragment, { key: index }, [
            props.metadata.container.wrap ? (openBlock(), createElementBlock("div", {
              key: 0,
              class: normalizeClass(unref(wrapClassName))
            }, [
              (openBlock(), createBlock(resolveDynamicComponent(unref(components2)[item.type]), {
                metadata: item,
                dataBinding: __props.dataBinding
              }, null, 8, ["metadata", "dataBinding"]))
            ], 2)) : (openBlock(), createBlock(resolveDynamicComponent(unref(components2)[item.type]), {
              key: 1,
              metadata: item,
              dataBinding: __props.dataBinding
            }, null, 8, ["metadata", "dataBinding"]))
          ], 64);
        }), 128))
      ], 2);
    };
  }
});
const _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "SupermaticUi",
  props: {
    configuration: null,
    onInit: { type: Function },
    registrations: null
  },
  setup(__props) {
    const props = __props;
    const wrapClassName = ref("");
    if (props.configuration && props.configuration.type === "layout") {
      console.log("[SupermaticUi] configuration", props.configuration);
    } else {
      console.log("[SupermaticUi] no configuration");
    }
    let schemaDefinition;
    let dataBinding;
    let registrations = props.registrations;
    let components2 = {};
    if (props.configuration != null && props.configuration.dataBinding != null && typeof props.configuration.dataBinding["type"] === "string") {
      schemaDefinition = props.configuration.dataBinding;
      dataBinding = dataBindingBuilder(
        {},
        schemaDefinition,
        registrations || createRegistrations({
          container: _sfc_main$1
        })
      );
      components2 = dataBinding.registrations.getComponentRegistrations();
      setupTemplate(dataBinding, props.configuration.layout.wrapClassName, wrapClassName);
      if (props.onInit) {
        props.onInit(dataBinding);
      }
    }
    console.log("[SupermaticUi] components", components2);
    return (_ctx, _cache) => {
      var _a;
      return openBlock(true), createElementBlock(Fragment, null, renderList((_a = props.configuration) == null ? void 0 : _a.layout.components, (item, index) => {
        var _a2;
        return openBlock(), createElementBlock(Fragment, { key: index }, [
          ((_a2 = props.configuration) == null ? void 0 : _a2.layout.wrap) ? (openBlock(), createElementBlock("div", {
            key: 0,
            class: normalizeClass(wrapClassName.value)
          }, [
            (openBlock(), createBlock(resolveDynamicComponent(unref(components2)[item.type]), {
              metadata: item,
              dataBinding: unref(dataBinding)
            }, null, 8, ["metadata", "dataBinding"]))
          ], 2)) : (openBlock(), createBlock(resolveDynamicComponent(unref(components2)[item.type]), {
            key: 1,
            metadata: item,
            dataBinding: unref(dataBinding)
          }, null, 8, ["metadata", "dataBinding"]))
        ], 64);
      }), 128);
    };
  }
});
const components = {
  SupermaticUi: _sfc_main,
  SupermaticContainer: _sfc_main$1
};
const UseSupermaticUi = {
  install: (app) => {
    for (let componentKey in components) {
      console.log("install component", componentKey);
      app.component(componentKey, components[componentKey]);
    }
  }
};
export {
  _sfc_main$1 as SupermaticContainer,
  _sfc_main as SupermaticUi,
  UseSupermaticUi,
  setupBind,
  setupEval,
  setupTemplate
};
